#!/bin/bash

# -----------------------------------------------------------------
# variables
# -----------------------------------------------------------------

. /etc/default/epages6

BN=${0##*/}
OPENSSL_CMD=openssl
INI_CONFIG=/var/epages/ini-config.sh

#mongo daemon name (used for stop/start)
#mongod_name() sourced by ../functions/fun-java.sh via /etc/default/epages6
MONGOD=$(mongod_name)

# argument defaults
MONGO_CONF_FILE=${MONGOD_CONF:-"/etc/$MONGOD.conf"}

# -----------------------------------------------------------------
# arguments
# -----------------------------------------------------------------

usage() {
  [[ $# -gt 0 ]] && echo -e "Error:\n  $@"
  echo "Usage:"
  echo "  $BN [-f keyfile] [-c mongodb.conf] [-h]"
  echo "Description:"
  echo "  create a keyfile to use for authenticate mongodb users"
  echo "Options:"
  echo "  -f: auth keyfile to create and use (default: <dbpath>/mongodb-keyfile)"
  echo "  -c: mongodb config file (default: $MONGO_CONF_FILE)"
  exit 1
}

#get options from command line
unset KEYFILE
while getopts c:f: char; do
case $char in
    f) KEYFILE=$OPTARG ;;
    c) MONGO_CONF_FILE=$OPTARG ;;
    *) usage ;;
  esac
done
shift `expr $OPTIND - 1`

[[ $# -ne 0 ]] && usage "$BN: No extra arguments allowed: $@"
[[ ! -f "$MONGO_CONF_FILE" ]] && usage "$BN: No such file: $MONGO_CONF_FILE"

if [[ -n $KEYFILE ]] ; then
  KEYFILE_DIR=$(dirname $KEYFILE)
else
  KEYFILE_DIR=$($INI_CONFIG -f $MONGO_CONF_FILE -k dbpath -G)
  [[ -z $KEYFILE_DIR ]] && { echo "Error: dbpath not found in $MONGO_CONF_FILE"; exit 1; }
  [[ -d $KEYFILE_DIR ]] || { echo "Error: dbpath: no such directory: $KEYFILE_DIR"; exit 1; }
  KEYFILE="$KEYFILE_DIR/mongodb-keyfile"
fi

#check if openssl is installed
type $OPENSSL_CMD >/dev/null || exit 1

# -----------------------------------------------------------------
# create keyfile
# -----------------------------------------------------------------

mkdir -m 755 -p "$KEYFILE_DIR" || exit 1
$OPENSSL_CMD rand -base64 741 > "$KEYFILE" || exit 1
chmod 600 "$KEYFILE"
chown $(find "$KEYFILE_DIR" -maxdepth 0 -printf '%u:%g') "$KEYFILE"

# -----------------------------------------------------------------
# add keys 'keyFile' and 'authorization' to mongod.conf
# -----------------------------------------------------------------

$INI_CONFIG -f $MONGO_CONF_FILE -k keyFile -v "$KEYFILE" -i || exit 1
$INI_CONFIG -f $MONGO_CONF_FILE -k auth -v true -i

exit $?
